package frames;

import javax.swing.*;
import javax.swing.border.StrokeBorder;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Objects;

import drawing_objects.*;
import generics.*;
import movement_strategy.*;

class TxtFilter extends FileFilter {
    @Override
    public boolean accept(File f) {
        if (f.isDirectory()) {
            return false;
        }
        String s = f.getName().toLowerCase();
        return s.endsWith(".txt");
    }
    @Override
    public String getDescription() {
        return "*.txt";
    }
}
public class FrameShipsCollection extends JFrame {
    private final ShipsGenericStorage storage;
    LinkedList<DrawingShip> trashCollection = new LinkedList<>();
    private JComponent pictureBoxCollection;
    private TextField textFieldNumber;
    private TextField textFieldStorageName;
    private JList<String> listStorages;
    private DefaultListModel<String> listModel;
    public FrameShipsCollection(){
        super("Набор кораблей");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        createGui();
        pack();
        setVisible(true);
        storage = new ShipsGenericStorage(pictureBoxCollection.getWidth(), pictureBoxCollection.getHeight());
    }
    private void createGui(){
        //components initialisation
        pictureBoxCollection = new JComponent(){
            public void paintComponent(Graphics graphics){
                super.paintComponent(graphics);
                Graphics2D graphics2D = (Graphics2D) graphics;
                if (listStorages == null || storage == null)
                    return;
                var collection = storage.getSet(listStorages.getSelectedValue());
                if (collection == null)
                    return;
                collection.showShips(graphics2D);
                super.repaint();
            }
        };
        JMenuBar menuFile = new JMenuBar();
        JMenu file = new JMenu("Файл");
        menuFile.add(file);
        JMenuItem saveFile = new JMenuItem("Сохранить");
        JMenuItem loadFile = new JMenuItem("Загрузить");
        JMenuItem saveSet = new JMenuItem("Сохранить коллекцию");
        JMenuItem loadSet = new JMenuItem("Загрузить коллекцию");
        file.add(saveSet);
        file.add(loadSet);
        file.add(saveFile);
        file.add(loadFile);
        setJMenuBar(menuFile);
        pictureBoxCollection.setPreferredSize(new Dimension(700, 450));
        JButton buttonAddShip = new JButton("Добавить корабль");
        textFieldNumber = new TextField();
        textFieldNumber.setText("0");
        JButton buttonRemoveShip = new JButton("Удалить корабль");
        JButton buttonRefreshCollection = new JButton("Обновить коллекцию");
        JButton buttonAddSet = new JButton("Добавить набор");
        JButton buttonDeleteSet = new JButton("Удалить набор");
        JButton buttonTrash = new JButton("Корзина");
        textFieldStorageName = new TextField();
        listModel = new DefaultListModel<>();
        JScrollPane scrollPane = new JScrollPane();
        listStorages= new JList<>(listModel);
        scrollPane.setViewportView(listStorages);
        //ActionListeners and ActionCommand addition
        buttonAddSet.addActionListener(e -> buttonAddSet_Click());
        buttonDeleteSet.addActionListener(e -> buttonDeleteSet_Click());
        buttonAddShip.addActionListener(e -> buttonAddShipClick());
        buttonRemoveShip.addActionListener(e -> buttonRemoveShipClick());
        buttonRefreshCollection.addActionListener(e -> buttonRefreshCollectionClick());
        buttonTrash.addActionListener(e -> buttonTrashClick());
        saveFile.addActionListener(e -> saveFile());
        saveSet.addActionListener(e -> saveSet());
        loadFile.addActionListener(e -> loadFile());
        loadSet.addActionListener(e -> loadSet());
        //panels and constraints initialisation
        JPanel panelTools = new JPanel(new GridBagLayout());
        panelTools.setBorder(new StrokeBorder(new BasicStroke(3)));
        panelTools.setToolTipText("Инструменты");
        JPanel panelSets = new JPanel(new GridBagLayout());
        panelSets.setBorder(new StrokeBorder(new BasicStroke(3)));
        panelSets.setToolTipText("Наборы");
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.insets.left = constraints.insets.right = 5;
        constraints.insets.top = constraints.insets.bottom = 5;
        constraints.fill = GridBagConstraints.BOTH;
        //addition to panelSets
        constraints.gridx = 0;
        constraints.gridy = 0;
        panelSets.add(textFieldStorageName, constraints);
        constraints.gridx = 0;
        constraints.gridy = 1;
        panelSets.add(buttonAddSet, constraints);
        constraints.gridx = 0;
        constraints.gridy = 2;
        panelSets.add(scrollPane, constraints);
        constraints.gridx = 0;
        constraints.gridy = 3;
        panelSets.add(buttonDeleteSet, constraints);
        //addition to panelTools
        constraints.gridx = 0;
        constraints.gridy = 0;
        panelTools.add(panelSets, constraints);
        constraints.gridx = 0;
        constraints.gridy = 1;
        panelTools.add(buttonAddShip, constraints);
        constraints.gridx = 0;
        constraints.gridy = 2;
        panelTools.add(textFieldNumber, constraints);
        constraints.gridx = 0;
        constraints.gridy = 3;
        panelTools.add(buttonRemoveShip, constraints);
        constraints.gridx = 0;
        constraints.gridy = 4;
        panelTools.add(buttonRefreshCollection, constraints);
        constraints.gridx = 0;
        constraints.gridy = 5;
        panelTools.add(buttonTrash, constraints);
        //addition to frame
        setLayout(new BorderLayout());
        add(panelTools, BorderLayout.EAST);
        add(pictureBoxCollection, BorderLayout.CENTER);
    }
    private void reloadObjects(){
        int index = listStorages.getSelectedIndex();
        listModel.clear();
        ArrayList<String> keys = storage.getKeys();
        for (String key : keys) {
            listModel.addElement(key);
        }
        if(listModel.size() > 0 && (index == -1 || index >= listModel.size()))
            listStorages.setSelectedIndex(0);
        else if(listModel.size() > 0)
            listStorages.setSelectedIndex(index);
    }
    private void buttonAddSet_Click() {
        if(textFieldStorageName.getText() == null ) {
            JOptionPane.showMessageDialog(this, "Не все данные заполнены");
            return;
        }
        String name = textFieldStorageName.getText();
        if (Objects.equals(name, "")) {
            JOptionPane.showMessageDialog(this, "Не все данные заполнены");
            return;
        }
        storage.addSet(name);
        reloadObjects();
    }
    private void buttonDeleteSet_Click() {
        if (listStorages.getSelectedIndex() == -1)
            return;
        storage.delSet(listStorages.getSelectedValue(), trashCollection);
        reloadObjects();
    }
    private void buttonAddShipClick() {
        if(listStorages.getSelectedIndex() == -1) {
            return;
        }
        ShipsGenericCollection<DrawingShip, DrawingObjectShip> drawingShips = storage.getSet(listStorages.getSelectedValue());
        FrameShipConfig frameShipConfig = new FrameShipConfig();
        frameShipConfig.addButton.addActionListener(e -> {
            if (drawingShips.insert(frameShipConfig.drawingShip)) {
                frameShipConfig.dispose();
                frameShipConfig.drawingShip.pictureWidth = pictureBoxCollection.getWidth();
                frameShipConfig.drawingShip.pictureHeight = pictureBoxCollection.getHeight();
                JOptionPane.showMessageDialog(null, "Объект добавлен", "Информация", JOptionPane.INFORMATION_MESSAGE);
                pictureBoxCollection.repaint();
            }
            else {
                JOptionPane.showMessageDialog(null, "Не удалось добавить объект", "Информация", JOptionPane.INFORMATION_MESSAGE);
            }
        });
        frameShipConfig.cancelButton.addActionListener(e -> frameShipConfig.dispose());
    }
    private void buttonRemoveShipClick(){
        if (listStorages.getSelectedIndex() == -1 || Objects.equals(textFieldNumber.getText(), "") || textFieldNumber.getText() == null)
            return;
        var obj = storage.getSet(listStorages.getSelectedValue());
        if (obj == null)
            return;
        int pos = Integer.parseInt(textFieldNumber.getText());
        var ship = obj.get(pos);
        if (obj.remove(pos)){
            JOptionPane.showMessageDialog(this, "Объект удален");
            trashCollection.add(ship);
            pictureBoxCollection.repaint();
        }
        else{
            JOptionPane.showMessageDialog(this, "Не удалось удалить объект");
        }
    }
    private void buttonRefreshCollectionClick(){
        pictureBoxCollection.repaint();
    }
    private void buttonTrashClick(){
        if (trashCollection.size() == 0)
            return;
        FrameBattleship frame;
        try {
            frame = new FrameBattleship();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        frame.setShip(trashCollection.pop());
    }
    private void saveFile(){
        JFileChooser fc = new JFileChooser("C:\\Users\\user\\Documents");
        fc.addChoosableFileFilter(new TxtFilter());
        int retrieval = fc.showSaveDialog(null);
        if (retrieval == JFileChooser.APPROVE_OPTION) {
            File file = new File(fc.getSelectedFile() + "." + "txt");
            try {
                storage.saveData(file);
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }
    private void saveSet(){
        JFileChooser fc = new JFileChooser("C:\\Users\\user\\Documents");
        fc.addChoosableFileFilter(new TxtFilter());
        int retrieval = fc.showSaveDialog(null);
        if (retrieval == JFileChooser.APPROVE_OPTION) {
            File file = new File(fc.getSelectedFile() + "." + "txt");
            try {
                if(listStorages.getSelectedIndex() == -1) {
                    return;
                }
                storage.shipsStorages.get(listStorages.getSelectedValue()).saveData(file, listStorages.getSelectedValue());
                reloadObjects();
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }
    private void loadFile(){
        JFileChooser fc = new JFileChooser("C:\\Users\\user\\Documents");
        fc.addChoosableFileFilter(new TxtFilter());
        int ret = fc.showDialog(null, "Открыть файл");
        if(ret == JFileChooser.APPROVE_OPTION){
            File file = fc.getSelectedFile();
            try {
                storage.loadData(file);
                reloadObjects();
                pictureBoxCollection.repaint();
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }
    private void loadSet(){
        JFileChooser fc = new JFileChooser("C:\\Users\\user\\Documents");
        int ret = fc.showDialog(null, "Открыть файл");
        fc.addChoosableFileFilter(new TxtFilter());
        if(ret == JFileChooser.APPROVE_OPTION){
            File file = fc.getSelectedFile();
            try {
                storage.loadCollection(file);
                reloadObjects();
                pictureBoxCollection.repaint();
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }
}
