package generics;

import drawing_objects.*;
import movement_strategy.*;

import java.awt.*;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

public class ShipsGenericCollection<T extends DrawingShip, U extends IMoveableObject>{
    private final int pictureWidth;
    private final int pictureHeight;
    private final int placeSizeWidth = 220;
    private final int placeSizeHeight = 60;
    public static char separatorRecords = ';';
    public static char separatorForObject = ':';
    private SetGeneric<T> collection;
    public ShipsGenericCollection(int picWidth, int picHeight){
        int width = picWidth / placeSizeWidth;
        int height = picHeight / placeSizeHeight;
        pictureWidth = picWidth;
        pictureHeight = picHeight;
        collection = new SetGeneric<>(width * height);
    }
    public boolean insert ( T obj) {
        if (obj != null)
            return collection.insert(obj);
        return false;
    }
    public boolean remove (int pos){
        return collection.remove(pos);
    }
    public U getU(int pos){
        if(collection.get(pos) == null)
            return null;
        return (U)collection.get(pos).getMoveableObject();
    }
    public T get(int position){
        if(position < 0 || position >= collection.getCount())
            return null;
        return collection.get(position);
    }
    public int getCount(){
        return collection.getCount();
    }
    public void showShips(Graphics2D graphics2D){
        drawBackground(graphics2D);
        drawObjects(graphics2D);
    }
    private void drawBackground(Graphics2D g){
        BasicStroke stroke = new BasicStroke(3);
        g.setStroke(stroke);
        for (int i = 0; i < pictureWidth / placeSizeWidth; i++){
            for (int j = 0; j < pictureHeight / placeSizeHeight + 1; ++j){
                g.drawLine(i * placeSizeWidth, j * placeSizeHeight, i *
                        placeSizeWidth + placeSizeWidth / 2, j * placeSizeHeight);
            }
            g.drawLine(i * placeSizeWidth, 0, i * placeSizeWidth,
                    pictureHeight / placeSizeHeight * placeSizeHeight);
        }
    }
    private void drawObjects(Graphics2D g){
        for (int i = 0; i < collection.getCount(); i++){
            DrawingShip ship = collection.get(i);
            if (ship != null)
            {
                int inRow = pictureWidth / placeSizeWidth;
                ship.setPosition(i % inRow * placeSizeWidth, pictureHeight - pictureHeight % placeSizeHeight - (i / inRow + 1) * placeSizeHeight + 5);
                ship.drawTransport(g);
            }
        }
    }
    public boolean saveData(File f, String name) throws IOException {
        if(f.exists()) {
            f.delete();
        }
        f.createNewFile();
        StringBuilder data = new StringBuilder();
        data.append("ShipsCollection\n");
        data.append(String.format("%s\n", name));
        StringBuilder records = new StringBuilder();
        for(DrawingShip elem : getShips()) {
            records.append(String.format("%s%c", ExtensionDrawingShip.getDataForSave(elem, separatorForObject), separatorRecords));
        }
        data.append(records);
        if(data.length() == 0)
            return false;
        FileWriter writer = new FileWriter(f);
        writer.write(data.toString());
        writer.flush();
        writer.close();
        return true;
    }
    public ArrayList<T> getShips(){
        return collection.getShips();
    }
    public void clear(){
        collection = new SetGeneric<>(pictureWidth * pictureHeight);
    }
}
