package generics;

import drawing_objects.DrawingShip;
import drawing_objects.ExtensionDrawingShip;
import movement_strategy.DrawingObjectShip;

import java.io.*;
import java.util.*;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public class ShipsGenericStorage {
    private final int pictureWidth;
    private final int pictureHeight;
    public final HashMap<String, ShipsGenericCollection<DrawingShip, DrawingObjectShip>> shipsStorages;
    private static final char separatorForKeyValue = '|';
    private final char separatorRecords = ';';
    private static final char separatorForObject = ':';
    public ShipsGenericStorage(int pictureWidth, int pictureHeight){
        shipsStorages = new HashMap<>();
        this.pictureWidth = pictureWidth;
        this.pictureHeight = pictureHeight;
    }
    public ArrayList<String> getKeys(){
        return new ArrayList<>(shipsStorages.keySet());
    }
    public void addSet(String name){
        if(shipsStorages.containsKey(name))
            return;
        shipsStorages.put(name, new ShipsGenericCollection<>(pictureWidth, pictureHeight));
    }
    public ShipsGenericCollection<DrawingShip, DrawingObjectShip> getSet(String name){
        if(!shipsStorages.containsKey(name))
            return null;
        return shipsStorages.get(name);
    }
    public DrawingShip getShip(String collectionName, int position){
        return shipsStorages.get(collectionName).get(position);
    }
    public boolean saveData(File f) throws IOException {
        if(f.exists()) {
            f.delete();
        }
        f.createNewFile();
        StringBuilder data = new StringBuilder();
        data.append("ShipStorage\n");
        for(Map.Entry<String, ShipsGenericCollection<DrawingShip, DrawingObjectShip>> record : shipsStorages.entrySet()){
            StringBuilder records = new StringBuilder();
            for(DrawingShip elem : record.getValue().getShips()) {
                records.append(String.format("%s%c", ExtensionDrawingShip.getDataForSave(elem, separatorForObject), separatorRecords));
            }
            data.append(String.format("%s%c%s\n", record.getKey(), separatorForKeyValue, records));
        }
        if(data.length() == 0)
            return false;
        FileWriter writer = new FileWriter(f);
        writer.write(data.toString());
        writer.flush();
        writer.close();
        return true;
    }
    public boolean loadData(File f) throws FileNotFoundException {
        if(!f.exists())
            return false;
        StringBuilder bufferTextFromFile =new StringBuilder();
        Scanner s = new Scanner(f);
        while(s.hasNext())
            bufferTextFromFile.append(s.next()).append("\n");
        s.close();
        var strs = bufferTextFromFile.toString().split("\n");
        if(strs.length == 0)
            return false;
        if (!strs[0].startsWith("ShipStorage"))
            return false;
        shipsStorages.clear();
        for(String data : strs){
            String st = "\\" + separatorForKeyValue;
            String[]record = data.split(st);
            if (record.length != 2)
                continue;
            ShipsGenericCollection<DrawingShip, DrawingObjectShip> collection = new ShipsGenericCollection<>(pictureWidth, pictureHeight);
            String[] set = record[1].split(Character.toString(separatorRecords));
            for(int i = set.length -1; i >=0; i--){
                String elem = set[i];
                DrawingShip ship = ExtensionDrawingShip.createDrawingShip(elem, separatorForObject, pictureWidth, pictureHeight);
                if (ship != null)
                {
                    if (!(collection.insert(ship)))
                    {
                        return false;
                    }
                }
            }
            shipsStorages.put(record[0], collection);
        }
        return true;
    }
    public boolean loadCollection(File f) throws FileNotFoundException {
        if(!f.exists())
            return false;
        StringBuilder bufferTextFromFile =new StringBuilder();
        Scanner s = new Scanner(f);
        while(s.hasNext())
            bufferTextFromFile.append(s.next()).append("\n");
        s.close();
        var strs = bufferTextFromFile.toString().split("\n");
        if(strs.length == 0)
            return false;
        if (!strs[0].startsWith("ShipsCollection"))
            return false;
        String collectionName = strs[1];
        ShipsGenericCollection<DrawingShip, DrawingObjectShip> collection = getCollection(collectionName);
        if(collection == null)
            collection = new ShipsGenericCollection<>(pictureWidth, pictureHeight);
        else
            collection.clear();
        String[] shipsInfo = strs[2].split(Character.toString(ShipsGenericCollection.separatorRecords));
        for(int i = shipsInfo.length-1; i >= 0; i--){
            String data = shipsInfo[i];
            DrawingShip ship = ExtensionDrawingShip.createDrawingShip(data, ShipsGenericCollection.separatorForObject, pictureWidth, pictureHeight);
            if (ship != null)
            {
                if (!(collection.insert(ship)))
                {
                    return false;
                }
            }
        }
        addSetFromFile(collectionName, collection);
        return true;
    }
    public void addSetFromFile(String name, ShipsGenericCollection<DrawingShip, DrawingObjectShip> toAdd){
        if(shipsStorages.containsKey(name)){
            shipsStorages.remove(name);
        }
        shipsStorages.put(name, toAdd);
    }
    public void delSet(String name, LinkedList<DrawingShip> trashBox){
        if(!shipsStorages.containsKey(name))
            return;
        ShipsGenericCollection<DrawingShip, DrawingObjectShip> cur = shipsStorages.get(name);
        for(int i = 0; i < cur.getCount(); i++)
            trashBox.push(cur.get(i));
        shipsStorages.remove(name);
    }
    public ShipsGenericCollection<DrawingShip, DrawingObjectShip> getCollection(String collectionName){
        return shipsStorages.get(collectionName);
    }
}
